import json
import time
from colorama import init, Fore
from prettytable import PrettyTable
from hpass.encryption import random_password, encryption_rc4, decrypt_rc4

init(autoreset=True)


class HPassCli:
    def __init__(self, primary, hello_password_data_dir):
        self.__primary = primary
        self.hello_password_data_dir = hello_password_data_dir
        with open(hello_password_data_dir, 'r', encoding='utf-8') as f:
            password_data_json = json.load(f)
        self.__password_data_json = password_data_json

    def save_data_file(self):
        with open(self.hello_password_data_dir, 'w', encoding='utf-8') as f:
            json.dump(self.__password_data_json, f, indent=4, ensure_ascii=False)
        return

    @staticmethod
    def get_random_password(length):
        try:
            _password_length = int(length)
            print(Fore.GREEN + random_password(length=_password_length))
        except ValueError:
            print(Fore.RED + 'The parameter `Length` requires a number ' + Fore.RESET + '(E.g random 16)')
        return

    def get_password_list(self):
        pt_able = PrettyTable('ID Website Notes Username Email Phone'.split(' '))
        for k, v in self.__password_data_json['account'].items():
            _data = decrypt_rc4(key=self.__primary, message=v)
            _data_dict = json.loads(_data)
            _data_list = [_data_dict['id'], _data_dict['website'], _data_dict['notes'], _data_dict['username'],
                          _data_dict['email'], _data_dict['phone']]
            pt_able.add_row(_data_list)
        print(pt_able)
        return

    def del_password(self, key):
        try:
            _message = self.__password_data_json['account'][key]
            print(Fore.MAGENTA + 'Please confirm that the operation target is this print content. Press Y/N')
            _data = decrypt_rc4(key=self.__primary, message=_message)
            _data_dict = json.loads(_data)
            print(Fore.CYAN + _data_dict['website'])
            print(Fore.CYAN + _data_dict['notes'])
            user_input = input('please enter: ')
            if user_input == 'Y' or user_input == 'y':
                del self.__password_data_json['account'][key]
                print(Fore.GREEN + 'Password successfully deleted !')
                self.save_data_file()
        except KeyError:
            print(Fore.RED + 'Password data not found')
        return

    def get_password(self, key):
        try:
            _message = self.__password_data_json['account'][key]
            _data = decrypt_rc4(key=self.__primary, message=_message)
            _data_dict = json.loads(_data)
            del _data_dict['id']
            del _data_dict['time']
            print(json.dumps(_data_dict, sort_keys=True, indent=4))
        except KeyError:
            print(Fore.RED + 'Password data not found')
        return

    def set_password(self, key, set_key):
        try:
            _message = self.__password_data_json['account'][key]
            _data = decrypt_rc4(key=self.__primary, message=_message)
            _data_dict = json.loads(_data)
            if set_key in _data_dict.keys():
                print(Fore.MAGENTA + 'Original ' + set_key + ' = ' + _data_dict[set_key])
                set_value_input = input('Now ' + set_key + ' = ')
                _data_dict[set_key] = set_value_input
                _now_password_str = json.dumps(_data_dict)
                _now_password_encryption = encryption_rc4(key=self.__primary, message=_now_password_str)
                self.__password_data_json['account'][key] = _now_password_encryption
                self.save_data_file()
                print(Fore.GREEN + 'Password value modified successfully!')
            else:
                print(Fore.RED + 'Password data does not have this value')
        except KeyError:
            print(Fore.RED + 'Password data not found')
        return

    def add_password(self):
        print(Fore.MAGENTA + 'The following is the information required for the new password :')
        website_input = input('Website = ')
        notes_input = input('Notes = ')
        username_input = input('Username = ')
        email_input = input('Email = ')
        phone_input = input('Phone = ')
        password_input = input('Password = ')
        new_password_dict = {
            'id': self.__password_data_json['gradual'],
            'website': website_input.strip(),
            'notes': notes_input.strip(),
            'username': username_input.strip(),
            'email': email_input.strip(),
            'phone': phone_input.strip(),
            'password': password_input.strip(),
            'time': time.time()
        }
        self.__password_data_json['gradual'] += 1
        _new_password_str = json.dumps(new_password_dict)
        _new_password_encryption = encryption_rc4(key=self.__primary, message=_new_password_str)
        self.__password_data_json['account'][new_password_dict['id']] = _new_password_encryption
        print(Fore.GREEN + 'The new password has been successfully added!')
        self.save_data_file()
        return


def cli_start(primary, hello_password_data_dir):
    h_pass_cli = HPassCli(primary=primary, hello_password_data_dir=hello_password_data_dir)
    while True:
        user_input = input('H-Pass> ')
        if user_input == 'exit' or user_input == 'quit':
            break
        else:
            if user_input == 'filepath':
                print(h_pass_cli.hello_password_data_dir)
            elif user_input == 'list':
                h_pass_cli.get_password_list()
            elif user_input == 'add':
                h_pass_cli.add_password()
            elif 'random' in user_input:
                user_input_list = user_input.split(' ')
                if len(user_input_list) != 2:
                    print('You may have to enter: ' + Fore.BLUE + 'random 16')
                    continue
                _length = user_input_list[1]
                if _length == '':
                    print(Fore.RED + 'Missing parameter `Length` ' + Fore.RESET + '(E.g random 16)')
                else:
                    h_pass_cli.get_random_password(length=_length)
            elif 'get' in user_input:
                user_input_list = user_input.split(' ')
                if len(user_input_list) != 2:
                    print('You may have to enter: ' + Fore.BLUE + 'get 10')
                    continue
                _key = user_input_list[1]
                if _key == '':
                    print(Fore.RED + 'Missing parameter `ID` ' + Fore.RESET + '(E.g get 10)')
                else:
                    h_pass_cli.get_password(key=_key)
            elif 'del' in user_input:
                user_input_list = user_input.split(' ')
                if len(user_input_list) != 2:
                    print('You may have to enter: ' + Fore.BLUE + 'del 10')
                    continue
                _key = user_input_list[1]
                if _key == '':
                    print(Fore.RED + 'Missing parameter `ID` ' + Fore.RESET + '(E.g del 10)')
                else:
                    h_pass_cli.del_password(key=_key)
            elif 'set' in user_input:
                user_input_list = user_input.split(' ')
                if len(user_input_list) != 3:
                    print('You may have to enter: ' + Fore.BLUE + 'set 10 notes')
                    continue
                _set = user_input_list[2]
                _key = user_input_list[1]
                if _key == '' or _set == '':
                    print(Fore.RED + 'Missing parameter `ID` ' + Fore.RESET + '(E.g set 10 notes)')
                else:
                    h_pass_cli.set_password(key=_key, set_key=_set)
            else:
                print(Fore.YELLOW + 'Is the instruction correct?')



import random
import hmac
import base64
from hashlib import sha256


def random_password(length):
    base_char = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'I', 'H', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S',
                 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'i', 'h', 'j', 'k', 'l',
                 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2', '3', '4',
                 '5', '6', '7', '8', '9', '!', '@', '#', '$', '%', '^', '&', '*']
    password = ''
    for j in range(length):
        m = random.randint(0, len(base_char) - 1)
        password = password + base_char[m]
    return password


def rc4_init_s_box(key):
    s_box = list(range(256))
    j = 0
    for i in range(256):
        j_step_1 = j + s_box[i]
        j_step_2 = j_step_1 + ord(key[i % len(key)])
        j_step_3 = j_step_2 % 256
        j = j_step_3
        s_box[i], s_box[j] = s_box[j], s_box[i]
    return s_box


def rc4_res_program(s_box, message):
    res = []
    i = j = 0
    for s in message:
        i = (i + 1) % 256
        j = (j + s_box[i]) % 256
        s_box[i], s_box[j] = s_box[j], s_box[i]
        t = (s_box[i] + s_box[j]) % 256
        k = s_box[t]
        res.append(chr(ord(s) ^ k))
    return res


def encryption_rc4(key, message):
    s_box = rc4_init_s_box(key)
    res = rc4_res_program(s_box, message)
    cipher = "".join(res)
    cipher_text = str(base64.b64encode(cipher.encode('utf-8')), 'utf-8')
    return cipher_text


def decrypt_rc4(key, message):
    s_box = rc4_init_s_box(key)
    plain = base64.b64decode(message.encode('utf-8'))
    plain = bytes.decode(plain)
    res = rc4_res_program(s_box, plain)
    cipher = "".join(res)
    return cipher


def hmac_sha256_digest(value, key='35a07615d258ae2e067879a472e050cc3716'):
    app_secret = key.encode('utf-8')
    data = value.encode('utf-8')
    signature = base64.b64encode(hmac.new(app_secret, data, digestmod=sha256).digest())
    return signature



import sys
import json
import getpass
import argparse
from pathlib import Path
from colorama import init, Fore
from hpass.hpass_cli import cli_start
from hpass.encryption import random_password, hmac_sha256_digest

init(autoreset=True)


def main():
    if len(sys.argv) == 1:
        sys.argv.append('--cli')
    parser = argparse.ArgumentParser(description='Hello Password')
    parser.add_argument('-v', '--version', help='View version information', action='version', version='%(prog)s v0.0.5')
    parser.add_argument('-r', '--random_password',
                        help='Randomly generate passwords containing uppercase and lowercase letters/numbers/symbols',
                        action='store', dest='password_length')
    parser.add_argument('-i', '--initialization',
                        help='Create or specify a password storage file in the current directory', action='store_true',
                        dest='init_switch')
    parser.add_argument('-c', '--cli', help='Start CLI Workbench', action='store_true', dest='cli_switch')
    args = parser.parse_args()

    if args.init_switch:
        _primary_password = getpass.getpass("Your primary password: ")
        _primary_password_repeat = getpass.getpass("Enter your primary password again: ")
        if _primary_password != _primary_password_repeat:
            print(Fore.RED + 'The password entered twice is different')
            return
        _primary = hmac_sha256_digest(value=_primary_password).decode('utf-8')
        config_dir = Path(__file__).resolve().parents[0] / 'config.json'
        hello_password_data_dir = str(Path.cwd() / 'helloPasswordData.json')
        config_json = {'primary': _primary, 'hello_password_data_dir': hello_password_data_dir}
        if Path(hello_password_data_dir).is_file():
            print(Fore.BLUE + 'Find the password storage file in the current directory')
        else:
            hello_password_data_json = {'gradual': 10, 'account': {}}
            with open(hello_password_data_dir, 'w', encoding='utf-8') as f:
                json.dump(hello_password_data_json, f, indent=4, ensure_ascii=False)
        with open(str(config_dir), 'w', encoding='utf-8') as f:
            json.dump(config_json, f, indent=4, ensure_ascii=False)
        print(Fore.GREEN + 'Password storage file initialized successfully')

    if args.password_length:
        try:
            _password_length = int(args.password_length)
            rp = random_password(length=_password_length)
            print(Fore.GREEN + rp)
        except ValueError:
            print(Fore.RED + 'The parameter `password_length` requires a number ' + Fore.RESET + '(E.g hpass -r 16)')

    if args.cli_switch:
        config_dir = Path(__file__).resolve().parents[0] / 'config.json'
        if config_dir.is_file():
            _primary_password = getpass.getpass("Your primary password: ")
            _primary = hmac_sha256_digest(value=_primary_password).decode('utf-8')
            with open(str(config_dir), 'r', encoding='utf-8') as f:
                config_json = json.load(f)
                if _primary != config_json['primary']:
                    print(Fore.RED + 'Primary password is incorrect')
                    return
            cli_start(primary=_primary_password, hello_password_data_dir=config_json['hello_password_data_dir'])
        else:
            print(Fore.MAGENTA + 'Hello Password is a simple password management tool')
            print(Fore.YELLOW + '  I need a new password storage file')
            print(Fore.CYAN + '    $ cd [Password storage file directory]')
            print(Fore.CYAN + '    $ hpass -i')
            print(Fore.CYAN + '    $ [Your primary password]')
            print(Fore.CYAN + '    $ [Enter your primary password again]')
            print(Fore.YELLOW + '  I already have a password storage file')
            print(Fore.CYAN + '    $ cd [Existing password storage file directory]')
            print(Fore.CYAN + '    $ hpass -i')
            print(Fore.CYAN + '    $ [Your primary password]')
            print(Fore.CYAN + '    $ [Enter your primary password again]')


if __name__ == '__main__':
    main()
